unit ClsRSS;

interface

uses System.Collections;

type
  TRSSItem = class
    public
      Title,
      Link,
      Description,
      Author,
      Comments,
      PubDate : string;
  end;

  TRSS = class

  private
    rssItems : ArrayList;
  public
    Title,
    Link,
    Description,
    Language,
    Copyright,
    ManagingEditor,
    WebMaster,
    PubDate,
    LastBuildDate,
    Generator,
    Docs,
    Img,
    ImgTitle,
    ImgLink,
    ImgURL : string;

    constructor Create(FileName : string); overload;
    function    SaveFeed(FileName : string) : Boolean;

    function    GetRSSItem(itemNo : integer) : TRSSItem;
    function    GetRSSItemCount : integer;
    procedure   DeleteRSSItem(itemNo : integer);
    procedure   AddRSSItem(RSSItem : TRSSItem);
    procedure   UpdateRSSItem(itemNo : integer; RSSItem : TRSSItem);
  end;

implementation

uses System.Xml;

(* _________________________________________________________________________ *)

procedure TRSS.AddRSSItem(rssItem: TRSSItem);
var newrssItem : TRSSItem;
begin
    newrssItem := TRSSItem.Create;

    newrssItem.Title       := rssItem.Title;
    newrssItem.Link        := rssItem.Link;
    newrssItem.Description := rssItem.Description;
    newrssItem.Author      := rssItem.Author;
    newrssItem.Comments    := rssItem.Comments;
    newrssItem.PubDate     := rssItem.PubDate;

    rssItems.Insert(0, newrssItem);
end;

(* _________________________________________________________________________ *)

constructor TRSS.Create(FileName : string);
var xmlDoc : XmlDocument;
    xmlNodes : XmlNodeList;
    Node : XmlNode;
    rssItem : TRSSItem;
    str : string;
    NodeEnum : IEnumerator;

    function __GetNode(nd : XmlNode;  xpath : string) : string;
    var Node : XmlNode;
    begin
    	Node := nd.SelectSingleNode(xpath);
      if Node <> nil then __GetNode := Node.InnerText
      else __GetNode := '';
    end;

begin
  inherited Create;

  xmlDoc := XmlDocument.Create;
  xmlDoc.Load(FileName);

	Title          := __GetNode(xmlDoc,'rss/channel/title');
	Link           := __GetNode(xmlDoc,'rss/channel/link');
	Description    := __GetNode(xmlDoc,'rss/channel/description');
	Language       := __GetNode(xmlDoc,'rss/channel/language');
	Copyright      := __GetNode(xmlDoc,'rss/channel/copyright');
	ManagingEditor := __GetNode(xmlDoc,'rss/channel/managingEditor');
	Language       := __GetNode(xmlDoc,'rss/channel/language');
	WebMaster      := __GetNode(xmlDoc,'rss/channel/webMaster');
	PubDate        := __GetNode(xmlDoc,'rss/channel/pubDate');
	LastBuildDate  := __GetNode(xmlDoc,'rss/channel/lastBuildDate');
	Generator      := __GetNode(xmlDoc,'rss/channel/generator');
	Docs           := __GetNode(xmlDoc,'rss/channel/docs');
	ImgTitle       := __GetNode(xmlDoc,'rss/channel/image/title');
	ImgLink        := __GetNode(xmlDoc,'rss/channel/image/link');
	ImgURL         := __GetNode(xmlDoc,'rss/channel/image/url');

  rssItems := ArrayList.Create;

  xmlNodes := xmlDoc.SelectNodes('rss/channel/item');

  NodeEnum := xmlNodes.GetEnumerator;
  while NodeEnum.MoveNext() do begin
    rssItem := TRSSItem.Create;
    Node := NodeEnum.Current As XmlNode;

    str:=Node.OuterXml;

    rssItem.Title       := __GetNode(Node,'title');
    rssItem.Link        := __GetNode(Node,'link');
    rssItem.Description := __GetNode(Node,'description');
    rssItem.Author      := __GetNode(Node,'author');
    rssItem.Comments    := __GetNode(Node,'comments');
    rssItem.PubDate     := __GetNode(Node,'pubDate');

    rssItems.Add(rssItem);
  end;
end;

(* _________________________________________________________________________ *)

procedure TRSS.DeleteRSSItem(itemNo: integer);
begin
  rssItems.RemoveAt(itemNo);
end;

(* _________________________________________________________________________ *)

function TRSS.GetRSSItem(itemNo: integer): TRSSItem;
begin
  GetRSSItem := rssItems.Item[itemNo] As TRSSItem;
end;

(* _________________________________________________________________________ *)

function TRSS.GetRSSItemCount: integer;
begin
  GetRSSItemCount := rssItems.Count;
end;

(* _________________________________________________________________________ *)

function TRSS.SaveFeed(FileName: string): Boolean;
var xmlDoc : XmlDocument;
    xmlDec : XmlDeclaration;
    xmlRoot, newNode, channelNode, imageNode : XmlNode;
    xmlAtt : XmlAttribute;
    itemNo : integer;

    procedure __AddElement(destNode : XmlNode; NodeName, NodeContent : string);
    begin
      newNode           := xmlDoc.CreateNode(XmlNodeType.Element, NodeName, '');
      newNode.InnerText := NodeContent;
	    destNode.AppendChild(newNode);
    end;

    procedure __AddRSSItem(rssItem : TRSSItem);
    var itemNode : XmlNode;
    begin
      itemNode := xmlDoc.CreateNode(XmlNodeType.Element, 'item', '');
      __AddElement(itemNode, 'title',       rssItem.Title);
      __AddElement(itemNode, 'link',        rssItem.Link);
      __AddElement(itemNode, 'description', rssItem.Description);
      __AddElement(itemNode, 'author',      rssItem.Author);
      __AddElement(itemNode, 'pubDate',     rssItem.PubDate);
      channelNode.AppendChild(itemNode)
    end;

begin
  xmlDoc := XmlDocument.Create;

  xmlDec :=  xmlDoc.CreateXmlDeclaration('1.0', '', '');
  xmlDoc.AppendChild(xmlDec);

  xmlRoot := xmlDoc.CreateElement('rss');
  xmlAtt  := xmlDoc.CreateAttribute('version');
  xmlAtt.InnerText := '2.0';
  xmlRoot.Attributes.Append(xmlAtt);
  xmlDoc.AppendChild(xmlRoot);

  channelNode := xmlDoc.CreateNode(XmlNodeType.Element, 'channel', '');

  __AddElement(channelNode, 'title',         Title);
  __AddElement(channelNode, 'link',          Link);
  __AddElement(channelNode, 'description',   Description);
  __AddElement(channelNode, 'language',      Language);
  __AddElement(channelNode, 'copyright',     Copyright);
  __AddElement(channelNode, 'managingEditor',ManagingEditor);
  __AddElement(channelNode, 'webMaster',     WebMaster);
  __AddElement(channelNode, 'pubDate',       PubDate);
  __AddElement(channelNode, 'generator',     Generator);
  __AddElement(channelNode, 'docs',          Docs);

  imageNode := xmlDoc.CreateNode(XmlNodeType.Element, 'image', '');
  __AddElement(imageNode, 'title', ImgTitle);
  __AddElement(imageNode, 'link',  ImgLink);
  __AddElement(imageNode, 'url',   ImgURL);
  channelNode.AppendChild(imageNode);

  for itemNo := 0 to GetRSSItemCount - 1 do
    __AddRSSItem( GetRSSItem(itemNo) );

  xmlRoot.AppendChild(channelNode);

  xmlDoc.Save(FileName);

  SaveFeed := True;
end;

procedure TRSS.UpdateRSSItem(itemNo: integer; RSSItem: TRSSItem);
begin
  with TRSSItem(rssItems[itemNo]) do begin
    Title       := RSSItem.Title;
    Link        := RSSItem.Link;
    Description := RSSItem.Description;
    Author      := RSSItem.Author;
    Comments    := RSSItem.Comments;
    PubDate     := RSSItem.PubDate;
  end;
end;

end.
